@@ -151,15 +151,3 @@ NULL
 #' @usage data(taxonomy_table_ITS)
 NULL
 
-
-#' The fasta file of 16S dataset used in tax4fun2 method.
-#'
-#' @description
-#' See the document of microtable class for more details. This file is with read.fasta function in seqinr package.
-#'
-#' @docType data
-#' @keywords list
-#' @name rep_fasta_16S
-#' @usage data(rep_fasta_16S)
-NULL
-
