@@ -2,7 +2,8 @@
 #' Create trans_classifier object for machine-learning-based model prediction.
 #'
 #' @description
-#' This class is a wrapper for methods of machine-learning-based classification models.
+#' This class is a wrapper for methods of machine-learning-based classification models, including data pre-processing, feature selection, 
+#' data split, model training, prediction, confusionMatrix and ROC (Receiver Operator Characteristic) or PR (Precision-Recall) curve.
 #'
 #' Author(s): Felipe Mansoldo and Chi Liu
 #'
@@ -14,11 +15,11 @@ trans_classifier <- R6::R6Class(classname = "trans_classifier",
 		#' 
 		#' @param dataset the object of \code{\link{microtable}} Class.
 		#' @param x.predictors default "all"; character string or data.frame; a character string represents selecting the corresponding data from microtable$taxa_abund; 
-		#'   data.frame represents other customized data. See the following available options and description:
+		#'   data.frame represents other customized data. See the following available options:
 		#'   \describe{
 		#'     \item{\strong{'all'}}{use all the taxa stored in microtable$taxa_abund}
-		#'     \item{\strong{'Genus'}}{use Genus level table in microtable$taxa_abund, or other specific taxonomic rank}
-		#'     \item{\strong{other input}}{must be a data.frame; It should be have the same format with the data.frame in microtable$taxa_abund, i.e. rows are features; 
+		#'     \item{\strong{'Genus'}}{use Genus level table in microtable$taxa_abund, or other specific taxonomic rank, e.g. 'Phylum'}
+		#'     \item{\strong{other input}}{must be a data.frame; It should have the same format with the data.frame in microtable$taxa_abund, i.e. rows are features; 
 		#'       cols are samples with same names in sample_table}
 		#'   }
 		#' @param y.response default NULL; the response variable in sample_table.
@@ -29,7 +30,7 @@ trans_classifier <- R6::R6Class(classname = "trans_classifier",
 		#' data(dataset)
 		#' t1 <- trans_classifier$new(
 		#' 		dataset = dataset, 
-		#' 		x.predictors = "all",
+		#' 		x.predictors = "Genus",
 		#' 		y.response = "Group")
 		#' }
 		initialize = function(dataset = NULL,
@@ -47,8 +48,8 @@ trans_classifier <- R6::R6Class(classname = "trans_classifier",
 			if ( nlevels(as.factor(sampleinfo[, y.response])) < 2 ) {
 				stop("Response variable must have at least 2 factors!")
 			}
-			# first judge whether x.predictors is character
-			if(is.character(class(x.predictors))){
+			# x.predictors must be character or data.frame
+			if(is.character(x.predictors)){
 				if (grepl("all", x.predictors, ignore.case = TRUE)) {
 					abund_table <- do.call(rbind, unname(dataset$taxa_abund))
 				}else{
@@ -56,7 +57,7 @@ trans_classifier <- R6::R6Class(classname = "trans_classifier",
 				}
 			}else{
 				# first check the data.frame
-				if(! is.data.frame(x.predictors)){
+				if(!is.data.frame(x.predictors)){
 					stop("Provided x.predictors is neither character nor data.frame !")
 				}
 				abund_table <- x.predictors
@@ -78,12 +79,11 @@ trans_classifier <- R6::R6Class(classname = "trans_classifier",
 			ClassNames <- make.names(sampleinfo[, y.response], unique = F)
 			MapNames <- data.frame(OriginalNames = sampleinfo[, y.response], ClassNames = ClassNames)
 			self$MapNames <- MapNames
-			DataX <- abund_table %>% t() %>% as.data.frame()
-			
+			DataX <- abund_table %>% t() %>% as.data.frame()			
 			message("Total feature numbers: ", ncol(DataX))
 			
 			if(all.equal(MapNames$OriginalNames, MapNames$ClassNames) != TRUE){
-				message("Factor names are non-standard. A correction was made and the change map was saved in object$MapNames")
+				message("Factor names are non-standard. A correction was made and the change map was saved in object$MapNames ...")
 			}
 			# message("Start clasification for: ", y.response, " ...")
 			if(n.cores > 1){
@@ -97,7 +97,25 @@ trans_classifier <- R6::R6Class(classname = "trans_classifier",
 			message("The response variable is stored in object$data_response ...")
 		},
 		#' @description
+		#' Pre-process (centering, scaling etc.) of the feature data based on the caret::preProcess function. 
+		#' 	 See \href{https://topepo.github.io/caret/pre-processing.html}{https://topepo.github.io/caret/pre-processing.html} for more details.
+		#' 
+		#' @param ... parameters pass to preProcess function of caret package.
+		#' @return converted data_feature in the object.
+		#' @examples
+		#' \dontrun{
+		#' t1$cal_preProcess(method = c("center", "scale", "nzv"))
+		#' }
+		cal_preProcess = function(...){
+			raw_feature <- self$data_feature
+			preProcess_res <- caret::preProcess(raw_feature, ...)
+			new_data <- predict(preProcess_res, newdata = raw_feature)
+			self$data_feature <- new_data
+			message("The converted feature table is stored in object$data_feature ...")
+		},
+		#' @description
 		#' Perform feature selection.
+		#' 	 See \href{https://topepo.github.io/caret/feature-selection-overview.html}{https://topepo.github.io/caret/feature-selection-overview.html} for more details.
 		#' 
 		#' @param boruta.maxRuns default 300; maximal number of importance source runs; passed to the maxRuns parameter in Boruta function of Boruta package.
 		#' @param boruta.pValue default 0.01; p value passed to the pValue parameter in Boruta function of Boruta package.
@@ -146,7 +164,7 @@ trans_classifier <- R6::R6Class(classname = "trans_classifier",
 		#' Split data for training and testing.
 		#' 
 		#' @param prop.train default 3/4; the ratio of the dataset used for the training.
-		#' @return optimized data_feature in the object.
+		#' @return data_train and data_test in the object.
 		#' @examples
 		#' \donttest{
 		#' t1$cal_split(prop.train = 3/4)
@@ -175,38 +193,47 @@ trans_classifier <- R6::R6Class(classname = "trans_classifier",
 			message("Training and testing data are stored in object$data_train and object$data_test respectively ...")
 		},
 		#' @description
-		#' Set trainControl for the following training.
+		#' Control parameters for the following training. See trainControl function of caret package for details.
 		#' 
-		#' @param method default 'repeatedcv'; the method used in trainControl function of caret package.
+		#' @param method default 'repeatedcv'; 'repeatedcv': Repeated k-Fold cross validation; 
+		#' 	 see method parameter in trainControl function of caret package for available options.
+		#' @param classProbs default TRUE; should class probabilities be computed for classification models?;
+		#' 	 see classProbs parameter in caret::trainControl function.
+		#' @param savePredictions default TRUE; see savePredictions parameter in caret::trainControl function
 		#' @param ... parameters pass to trainControl function of caret package.
 		#' @return trainControl in the object.
 		#' @examples
 		#' \dontrun{
-		#' t1$set_trainControl()
+		#' t1$set_trainControl(method = 'repeatedcv')
 		#' }
 		set_trainControl = function(
 			method = 'repeatedcv',
+			classProbs = TRUE,
+			savePredictions = TRUE,
 			...
 			){
 			trainControl <- caret::trainControl(method = method,
-								   classProbs = TRUE,
-								   savePredictions = TRUE,
+								   classProbs = classProbs,
+								   savePredictions = savePredictions,
 								   ...)
 			message('Generating trainControl setting stored in object$trainControl ...')
 			self$trainControl <- trainControl
 		},
 		#' @description
-		#' Run the training.
+		#' Run the model training.
 		#' 
-		#' @param method default "rf"; representing the random forest method; see method in train function of caret package.
-		#' @param metric default "Accuracy"; see metric in train function of caret package.
-		#' @param max.mtry default 2; maximum mtry.
-		#' @param max.ntree default 200; Number of trees to grow; pass to the ntree parameter of randomForest function in randomForest package.
+		#' @param method default "rf"; "rf": random forest; see method in caret::train function for other options.
+		#' @param metric default "Accuracy"; see metric in caret::train function for other options.
+		#' @param max.mtry default 2; for method = "rf"; maximum mtry used for the tunegrid to do hyperparameter tuning to optimize the model.
+		#' @param max.ntree default 200; for method = "rf"; maximum number of trees used to optimize the model.
 		#' @param ... parameters pass to train function of caret package.
 		#' @return res_train in the object.
 		#' @examples
 		#' \dontrun{
-		#' t1$cal_train()
+		#' # random forest
+		#' t1$cal_train(method = "rf")
+		#' # Support Vector Machines with Radial Basis Function Kernel
+		#' t1$cal_train(method = "svmRadial", tuneLength = 15)
 		#' }
 		cal_train = function(
 			method = "rf",
@@ -216,21 +243,19 @@ trans_classifier <- R6::R6Class(classname = "trans_classifier",
 			...
 			){
 			train_data <- self$data_train
-			control <- self$trainControl
+			trControl <- self$trainControl
 			
 			###################### ----------------
-			######################
 			if(method == "rf"){
 				# Optimization of RF parameters
-			  set.seed(12345)
 				message("Optimization of Random Forest parameters ...")
 
 				tunegrid <- expand.grid(.mtry=seq(from =1, to = max.mtry) )
 				modellist<- list()
 				#
 				for (ntree in c(100, max.ntree)) {
-					fit <- caret::train(Class~., data=train_data, method = method, metric=metric, 
-									  tuneGrid=tunegrid, trControl=control, ntree=ntree, ...)
+					fit <- caret::train(Class ~ ., data = train_data, method = method, metric=metric, 
+									  tuneGrid=tunegrid, trControl=trControl, ntree=ntree, ...)
 					key <- toString(ntree)
 					modellist[[key]] <- fit
 				}
@@ -240,31 +265,32 @@ trans_classifier <- R6::R6Class(classname = "trans_classifier",
 				res.tune1 <- as.data.frame(res.tune1$statistics$Accuracy)
 				#summary(results)
 
-				ntree = as.numeric(rownames(res.tune1)[which(res.tune1$Mean == max(res.tune1$Mean))])[1]
+				ntree <- as.numeric(rownames(res.tune1)[which(res.tune1$Mean == max(res.tune1$Mean))])[1]
 				#tunegrid <- expand.grid(.mtry=seq(from = 1, to=4, by = 0.5))
 				modellist <- list()
 
-				fit <- caret::train(Class~., data=train_data, method = method, 
-					metric=metric, tuneGrid=tunegrid, 
-					trControl=control, ntree = ntree)
+				fit <- caret::train(Class ~ ., data = train_data, method = method, 
+					metric = metric, tuneGrid = tunegrid, 
+					trControl = trControl, ntree = ntree)
 
 				message("ntree used:", ntree)
 				message("best mtry:", fit$bestTune$mtry)
 
 				tunegrid <- expand.grid(.mtry=fit$bestTune$mtry)
-				fit.best <- caret::train(x=train_data[,2:ncol(train_data)], y=train_data[,1], method= method, 
-										 metric=metric, tuneGrid=tunegrid, trControl=control, ntree=ntree, ...)
-				self$res_train <- fit.best
-				self$train_method <- method
-				message('The training result is stored in object$res_train ...')
-				
+				res_train <- caret::train(x=train_data[,2:ncol(train_data)], y = train_data[,1], method = method, 
+										 metric = metric, tuneGrid = tunegrid, trControl = trControl, ntree = ntree, ...)
+
 				######################Optimization of RF parameters end				
+			}else{
+				res_train <- caret::train(Class ~ ., data = train_data, method = method, trControl = trControl, ...)
 			}
-			###################### ----------------
+			self$res_train <- res_train
+			message('The training result is stored in object$res_train ...')
+			self$train_method <- method
 		},
 		#' @description
 		#' Get feature importance from the training model.
-		#' @param ... parameters pass to the evaluating function; If "rf" used, pass to randomForest::importance.
+		#' @param ... parameters pass to varImp function of caret package.
 		#' @return res_feature_imp in the object. One row for each predictor variable. The column(s) are different importance measures.
 		#' @examples
 		#' \dontrun{
@@ -274,17 +300,17 @@ trans_classifier <- R6::R6Class(classname = "trans_classifier",
 			if(is.null(self$res_train)){
 				stop("Please first run cal_train to train the model !")
 			}
-			if(self$train_method == "rf"){
-				res_feature_imp <- randomForest::importance(self$res_train$finalModel, ...)
-			}
+			res_feature_imp <- caret::varImp(self$res_train$finalModel, ...)
+			
 			self$res_feature_imp <- res_feature_imp
-			message('The feature importance evaluating result is stored in object$res_feature_imp ...')
+			message('The feature importance is stored in object$res_feature_imp ...')
 		},
 		#' @description
 		#' Run the prediction.
 		#' 
-		#' @param positive_class default NULL; see positive parameter in confusionMatrix function of caret package.
-		#' @return res_confusion_fit and res_confusion_stats stored in the object.
+		#' @param positive_class default NULL; see positive parameter in confusionMatrix function of caret package;
+		#' If positive_class is NULL, use the first group in data as the positive class automatically.
+		#' @return res_predict, res_confusion_fit and res_confusion_stats stored in the object.
 		#' @examples
 		#' \dontrun{
 		#' t1$cal_predict()
@@ -301,8 +327,8 @@ trans_classifier <- R6::R6Class(classname = "trans_classifier",
 			MapNames <- self$MapNames
 
 			fit.best.predict <- predict(fit.best, test_data[, 2:ncol(test_data)])
-			self$fit.best.predict <- fit.best.predict
-			message('The result of model prediction is stored in object$fit.best.predict ...')
+			self$res_predict <- fit.best.predict
+			message('The result of model prediction is stored in object$res_predict ...')
 
 			###################### 
 			######################    end: Evaluation for the test set
@@ -321,7 +347,6 @@ trans_classifier <- R6::R6Class(classname = "trans_classifier",
 
 			self$res_confusion_fit <- confusion.fit.best
 			message('The result of confusionMatrix is stored in object$res_confusion_fit ...')
-			
 			confusion.data.sts <- data.frame(confusion.fit.best$overall)
 			Confusion.Sts <- data.frame("Overall Statistics" = paste0(round(confusion.data.sts[,1],2) * 100,"%")  )
 			rownames(Confusion.Sts) <- rownames(confusion.data.sts)
@@ -334,7 +359,7 @@ trans_classifier <- R6::R6Class(classname = "trans_classifier",
 		#' 
 		#' @param plot_confusion default TRUE; whether plot the confusion matrix.
 		#' @param plot_statistics default TRUE; whether plot the statistics.
-		#' @return ggplot object in the object.
+		#' @return ggplot object.
 		#' @examples
 		#' \dontrun{
 		#' t1$plot_confusionMatrix()
@@ -352,95 +377,149 @@ trans_classifier <- R6::R6Class(classname = "trans_classifier",
 				geom_tile(aes(fill = log(Freq)), colour = "white") +
 				scale_fill_gradient(low = "white", high = "steelblue") +
 				geom_text(aes(x = Reference, y = Prediction, label = Freq)) +
-				theme(legend.position = "none")# +
+				theme(legend.position = "none")
 
 			Confusion.Sts <- self$res_confusion_stats
 
 			p2 <- gridExtra::tableGrob(Confusion.Sts)
-
 			if(plot_confusion == TRUE & plot_statistics == TRUE){
-				gridExtra::grid.arrange(p1, p2,nrow = 1, ncol = 2, 
+				p3 <- gridExtra::grid.arrange(p1, p2,nrow = 1, ncol = 2, 
 					top=grid::textGrob("Confusion Matrix and Statistics",gp=grid::gpar(fontsize=15,font=0.5)))
 			}
 			if(plot_confusion == TRUE & plot_statistics == FALSE){
-				gridExtra::grid.arrange(p1,nrow = 1, ncol = 1, 
+				p3 <- gridExtra::grid.arrange(p1,nrow = 1, ncol = 1, 
 					top=grid::textGrob("Confusion Matrix",gp=grid::gpar(fontsize=15,font=0.5)))
 			}
 			if(plot_confusion == FALSE & plot_statistics == TRUE){
-				gridExtra::grid.arrange(p2,nrow = 1, ncol = 1, 
+				p3 <- gridExtra::grid.arrange(p2,nrow = 1, ncol = 1, 
 					top=grid::textGrob("Statistics",gp=grid::gpar(fontsize=15,font=0.5)))
 			}
+			p3
 		},
 		#' @description
-		#' Get ROC curve data and the performance data.
+		#' Get ROC (Receiver Operator Characteristic) curve data and the performance data.
 		#' 
-		#' @param ... parameters pass to plot.performance function of ROCR package.
-		#' @return a list including res_perf, all_auc_perf and all_perf_table stored in the object.
+		#' @param input default "pred"; 'pred' or 'train'; 'pred' represents using prediction results;
+		#'   'train' represents using training results.
+		#' @return a list res_ROC stored in the object.
 		#' @examples
 		#' \dontrun{
 		#' t1$cal_ROC()
 		#' }
-		cal_ROC = function(...){
+		cal_ROC = function(input = "pred"){
+			input <- match.arg(input, c("pred", "train"))
 			if(is.null(self$res_train)){
 				stop("Please first run cal_train to train the model !")
 			}
-			test_data <- self$data_test
 			fit.best <- self$res_train
-
-			prediction_for_roc_curve <- predict(fit.best, test_data[, 2:ncol(test_data)] , type="prob")
-			classes <- levels(droplevels(test_data[, 1])) #drop because sometimes there is empty classes
-			# store the results
-			all_perf <- list()
-			all_auc_perf <- list()
-			all_perf_table <- data.frame()
-			res_ROC <- list()
+			train_method <- fit.best$method
 			
-			for(i in 1:length(classes)){
-			  # select observations of class[i]
-			  true_values <- ifelse(test_data[, 1] == classes[i], 1, 0)
-			  # performance for class[i]
-			  pred <- ROCR::prediction(prediction_for_roc_curve[, i], true_values)
-			  perf <- ROCR::performance(pred, "tpr", "fpr")
-			  all_perf[[i]] <- perf
-			  
-			  gg_df <- data.frame(x = attributes(perf)$x.values[[1]], y = attributes(perf)$y.values[[1]], Group = classes[i])
-			  all_perf_table <- rbind(all_perf_table, gg_df)
-			  #print(classes[i])
-			  auc.perf <- ROCR::performance(pred, measure = "auc")
-			  all_auc_perf[[i]] <- auc.perf
-			  #print(auc.perf@y.values)
-			  message('AUC of ', classes[i], " = ", auc.perf@y.values)
+			if(input == "pred"){
+				test_data <- self$data_test
+				prediction_prob <- predict(fit.best, test_data[, 2:ncol(test_data)] , type="prob")
+				class_names <- levels(droplevels(test_data[, 1])) #drop because sometimes there is empty classes
+				true_label <- test_data[, 1]
+			}else{
+				# use the prediction data in the training part
+				class_names <- fit.best$levels
+				prediction_prob <- fit.best$pred[, class_names]
+				true_label <- fit.best$pred$obs
 			}
-			res_ROC$res_perf <- all_perf
-			res_ROC$all_auc_perf <- all_auc_perf
-			res_ROC$all_perf_table <- all_perf_table
+			# use multiROC package
+			label_df <- lapply(class_names, function(x){ifelse(true_label == x, 1, 0)}) %>% 
+				do.call(cbind, .) %>%
+				as.data.frame %>%
+				`colnames<-`(paste0(class_names, "_true"))
+			prob_df <- prediction_prob %>% `colnames<-`(paste0(colnames(.), "_pred_", train_method))
+			use_df <- cbind(label_df, prob_df)
+
+			roc_res <- multiROC::multi_roc(use_df, force_diag = T)
+			pr_res <- multiROC::multi_pr(use_df, force_diag = T)
+
+			plot_roc_df <- multiROC::plot_roc_data(roc_res)
+			plot_pr_df <- multiROC::plot_pr_data(pr_res)
+
+			# store the results
+			res_ROC <- list()
+			res_ROC$res_roc <- plot_roc_df
+			res_ROC$res_pr <- plot_pr_df
 			self$res_ROC <- res_ROC
-			message('Raw class performance result of TPR-FPR is stored in the list object$res_ROC$res_perf ...')
-			message('Class performance table of TPR-FPR is stored in object$res_ROC$all_auc_perf ...')
-			message('Class performance of AUC is stored in object$res_ROC$all_perf_table ...')
+			message('Specificity-sensitivity data is stored in object$res_ROC$res_roc ...')
+			message('Recall-Precision is stored in object$res_ROC$res_pr ...')
 		},
 		#' @description
 		#' Plot ROC curve.
 		#' 
+		#' @param plot_type default c("ROC", "PR")[1]; 'ROC' represents ROC (Receiver Operator Characteristic) curve; 
+		#'   'PR' represents PR (Precision-Recall) curve.
+		#' @param plot_group default "all"; 'all' represents all the classes in the model;
+		#' 	 'add' represents all adding micro-average and macro-average results, see 
+		#' 	 \href{https://scikit-learn.org/stable/auto_examples/model_selection/plot_roc.html}{https://scikit-learn.org/stable/auto_examples/model_selection/plot_roc.html};
+		#' 	 other options should be one or more class names, same with the names in Group column of res_ROC$res_roc from cal_ROC function.
 		#' @param color_values default RColorBrewer::brewer.pal(8, "Dark2"); colors used in the plot.
-		#' @param ... parameters pass to geom_line function of ggplot2 package.
+		#' @param add_AUC default TRUE; whether add AUC in the legend.
+		#' @param plot_method default FALSE; If TRUE, show the method in the legend though only one method is found.
+		#' @param ... parameters pass to geom_path function of ggplot2 package.
 		#' @return ggplot2 object.
 		#' @examples
 		#' \dontrun{
 		#' t1$plot_ROC(size = 1, alpha = 0.7)
 		#' }
-		plot_ROC = function(color_values = RColorBrewer::brewer.pal(8, "Dark2"), ...){
+		plot_ROC = function(
+			plot_type = c("ROC", "PR")[1],
+			plot_group = "all",
+			color_values = RColorBrewer::brewer.pal(8, "Dark2"), 			
+			add_AUC = TRUE,
+			plot_method = FALSE,
+			...
+			){
+			
 			if(is.null(self$res_ROC)){
 				stop("Please first run cal_ROC to get the data for ROC curve !")
 			}
-			all_perf_table <- self$res_ROC$all_perf_table
-			p <- ggplot(data = all_perf_table, aes(x = x, y = y, color = Group, group = Group)) +
-				geom_line(...) + 
-#				geom_ribbon(aes(x, ymin = 0, ymax = y)) +
+			plot_type <- match.arg(plot_type, c("ROC", "PR"))
+			if(plot_type == "ROC"){
+				plot_data <- self$res_ROC$res_roc
+			}else{
+				plot_data <- self$res_ROC$res_pr
+			}
+			if(plot_group != "add"){
+				if(plot_group == "all"){
+					plot_data %<>% .[! .$Group %in% c("Micro", "Macro"), ]
+				}else{
+					if(!any(plot_data$Group %in% plot_group)){
+						stop("Please input the correct plot_group !")
+					}else{
+						plot_data %<>% .[.$Group %in% plot_group, ]
+					}
+				}
+			}
+			if(add_AUC){
+				plot_data$Group <- paste0(plot_data$Group, "\n AUC = ", round(plot_data$AUC, 2))
+			}
+		
+			if(plot_type == "ROC"){
+				p <- ggplot(plot_data, aes(x = 1-Specificity, y = Sensitivity)) + 
+					geom_segment(aes(x = 0, y = 0, xend = 1, yend = 1), colour = 'grey', linetype = 'dashed')
+			}else{
+				p <- ggplot(plot_data, aes(x = Recall, y = Precision))
+			}
+			if(length(unique(plot_data$Method)) > 1){
+				p <- p + geom_path(aes(color = Group, linetype = Method), ...)
+			}else{
+				if(plot_method){
+					p <- p + geom_path(aes(color = Group, linetype = Method), ...)
+				}else{
+					p <- p + geom_path(aes(color = Group), ...)
+				}
+			}
+			p <- p + theme_bw() + 
+				coord_equal() +
+				xlim(0, 1) +
+				ylim(0, 1) +
 				scale_color_manual(values = color_values) +
-				xlab("False positive rate") +
-				ylab("True positive rate") +
-				theme_bw()
+				theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank()) +
+				theme(legend.title = element_blank())
 			p
 		}
 	),
